import datetime
import json
import random

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.shortcuts import get_object_or_404

from ..models import Appointment, AppointmentTypeLink
from ..views.notifications import get_filter_locations

# mix_color = (0, 166, 90)
mix_color = (155, 155, 155)


def get_random_color():
    r = lambda: random.randint(0, 255)
    return '#%02X%02X%02X' % ((r() + mix_color[0]) / 2, (r() + mix_color[1]) / 2, (r() + mix_color[2]) / 2)


RANDOM_COLORS = [get_random_color() for i in range(20)]


def build_duration(duration):
    number_of_hours = duration // 60
    minutes = duration - number_of_hours * 60
    return "{0:02d}:{1:02d}".format(number_of_hours, minutes)


@login_required
def events(request, date):
    appointments = Appointment.objects.filter(
        datetime_when__date=date,
        location__in=get_filter_locations(request.user),
        status=Appointment.APPOINTMENT_STATUS_SCHEDULED,
        visit__isnull=False).all()
    subjects = {}
    for i, appointment in enumerate(appointments):
        appointment_subject = appointment.visit.subject
        if appointment_subject.id not in subjects:
            # create subject
            subject = {
                'name': str(appointment_subject),
                'id': appointment_subject.id,
                'color': RANDOM_COLORS[i],
                'events': []
            }
            subjects[appointment_subject.id] = subject
        links = AppointmentTypeLink.objects.filter(appointment=appointment).all()
        for j, link in enumerate(links):
            link_when = link.date_when
            link_end = None
            if link_when is not None:
                link_when = link_when.replace(tzinfo=None)
                link_end = link_when + datetime.timedelta(minutes=link.appointment_type.default_duration)
            event = {
                'title': link.appointment_type.description,
                'duration': build_duration(link.appointment_type.default_duration),
                'subject': str(appointment_subject),
                'id': '{}-{}'.format(i, j),
                'link_id': link.id,
                'link_when': link_when,
                'link_who': link.worker_id,
                'link_end': link_end,
                'location': str(appointment.location),
                'flying_team_location': str(appointment.flying_team),
                'appointment_start': appointment.datetime_when.replace(tzinfo=None),
                'appointment_end': appointment.datetime_when.replace(tzinfo=None, hour=19, minute=0),
            }
            subject_events = subjects[appointment_subject.id]['events']
            subject_events.append(event)
    return JsonResponse({
        "data": subjects.values(),
    })


@login_required
def events_persist(request):
    event_links = json.loads(request.POST.get('events_to_persist'))
    for event_link in event_links:
        try:
            when = datetime.datetime.strptime(event_link['start'], '%Y-%m-%dT%H:%M:00')
        except ValueError:
            when = datetime.datetime.strptime(event_link['start'][0:-6], '%Y-%m-%dT%H:%M:00')
        worker_id = event_link['link_who']
        link_id = event_link['link_id']
        appointment_type_link = get_object_or_404(AppointmentTypeLink, pk=link_id)
        appointment_type_link.worker_id = worker_id
        appointment_type_link.date_when = when
        appointment_type_link.save()
    events_to_clear = json.loads(request.POST.get('events_to_clear'))
    for link_id in events_to_clear:
        appointment_type_link = get_object_or_404(AppointmentTypeLink, pk=link_id)
        appointment_type_link.worker_id = None
        appointment_type_link.date_when = None
        appointment_type_link.save()
    return JsonResponse({}, status=201)
