from django.contrib.auth.decorators import login_required
from django.http import JsonResponse

from ..models import Worker
from ..views.notifications import get_filter_locations


@login_required
def specializations(request):
    X = Worker.objects.filter(specialization__isnull=False).values_list('specialization').distinct()
    return JsonResponse({
        "specializations": [x[0] for x in X]
    })


@login_required
def units(request):
    X = Worker.objects.filter(unit__isnull=False).values_list('unit').distinct()
    return JsonResponse({
        "units": [x[0] for x in X]
    })


@login_required
def workers_for_daily_planning(request):
    workers = Worker.objects.filter(locations__in=get_filter_locations(request.user)).exclude(
        role=Worker.ROLE_CHOICES_SECRETARY).distinct()
    workers_list_for_json = []
    for worker in workers:
        worker_dict_for_json = {
            'id': worker.id,
            'title': u"{} ({})".format(unicode(worker), worker.get_role_display()[:1].upper()),
            'role': worker.get_role_display()
        }
        workers_list_for_json.append(worker_dict_for_json)
    return JsonResponse(workers_list_for_json, safe=False)
